@@ -23,48 +23,70 @@ This module provides the primary command line argument parser for
 the torrentfile package. The main_script function is automatically
 invoked when called from command line, and parses accompanying arguments.
 
-Functions:
-    main_script: process command line arguments and run program.
-    activate_logger: turns on debug mode and logging facility.
+Functions
+---------
+main_script :
+    process command line arguments and run program.
+activate_logger :
+    turns on debug mode and logging facility.
+
+Classes
+-------
+Config : class
+    controls logging configuration
+TorrentFileHelpFormatter : HelpFormatter
+    the command line help message formatter
 """
 
+import io
 import logging
 import sys
 from argparse import ArgumentParser, HelpFormatter
+from typing import Optional
 
-from torrentfile.commands import create, edit, info, magnet, recheck
+from torrentfile.commands import create, edit, info, magnet, rebuild, recheck
 from torrentfile.interactive import select_action
 from torrentfile.version import __version__ as version
 
 
-def activate_logger():
+class Config:
     """
-    Activate the builtin logging mechanism when passed debug flag from CLI.
+    Class the controls the logging configuration and output settings.
+
+    Controls the logging level, or whether to app should operate in quiet mode.
     """
-    logging.basicConfig(level=logging.INFO)
-    logger = logging.getLogger()
-    file_handler = logging.FileHandler(
-        "torrentfile.log", mode="a+", encoding="utf-8"
-    )
-    console_handler = logging.StreamHandler(stream=sys.stderr)
-    file_formatter = logging.Formatter(
-        "%(asctime)s %(levelno)s %(message)s",
-        datefmt="%m-%d %H:%M:%S",
-        style="%",
-    )
-    stream_formatter = logging.Formatter(
-        "%(asctime)s %(levelno)s %(message)s",
-        datefmt="%m-%d %H:%M:%S",
-        style="%",
-    )
-    file_handler.setFormatter(file_formatter)
-    console_handler.setFormatter(stream_formatter)
-    file_handler.setLevel(logging.INFO)
-    console_handler.setLevel(logging.INFO)
-    logger.setLevel(logging.INFO)
-    logger.addHandler(console_handler)
-    logger.addHandler(file_handler)
-    logger.debug("Debug: ON")
+
+    @staticmethod
+    def activate_quiet():
+        """
+        Activate quiet mode for the duration of the programs life.
+
+        When quiet mode is enabled, no logging, progress or state information
+        is output to the terminal
+        """
+        if sys.stdout or sys.stderr:
+            sys.stdout = io.StringIO()
+            sys.stderr = io.StringIO()
+
+    @staticmethod
+    def activate_logger():
+        """
+        Activate the builtin logging mechanism when passed debug flag from CLI.
+        """
+        logging.basicConfig(level=logging.INFO)
+        logger = logging.getLogger()
+        console_handler = logging.StreamHandler(stream=sys.stderr)
+        stream_formatter = logging.Formatter(
+            "%(asctime)s %(levelno)s %(message)s",
+            datefmt="%m-%d %H:%M:%S",
+            style="%",
+        )
+        console_handler.setFormatter(stream_formatter)
+        console_handler.setLevel(logging.DEBUG)
+
+        logger.setLevel(logging.DEBUG)
+        logger.addHandler(console_handler)
+        logger.debug("Debug: ON")
 
 
 class TorrentFileHelpFormatter(HelpFormatter):
@@ -74,7 +96,7 @@ class TorrentFileHelpFormatter(HelpFormatter):
     Subclasses Argparse.HelpFormatter.
     """
 
-    def __init__(self, prog, width=40, max_help_positions=30):
+    def __init__(self, prog, width=45, max_help_positions=45):
         """
         Construct HelpFormat class for usage output.
 
@@ -87,9 +109,9 @@ class TorrentFileHelpFormatter(HelpFormatter):
         max_help_positions : int
             max length until line wrap.
         """
-        super().__init__(
-            prog, width=width, max_help_position=max_help_positions
-        )
+        super().__init__(prog,
+                         width=width,
+                         max_help_position=max_help_positions)
 
     def _split_lines(self, text, _):
         """
@@ -165,14 +187,23 @@ class TorrentFileHelpFormatter(HelpFormatter):
         return parts
 
 
-def execute(args=None):
+def execute(args: Optional[list] = None) -> list:
     """
-    Initialize Command Line Interface for torrentfile.
+    Execute program with provided list of arguments.
+
+    If no arguments are given then it defaults to using
+    sys.argv.  This is the main entrypoint for the program
+    and command line interface.
 
     Parameters
     ----------
     args : list
         Commandline arguments. default=None
+
+    Returns
+    -------
+    list
+        Depends on what the command line args were.
     """
     if not args:
         if sys.argv[1:]:
@@ -182,10 +213,10 @@ def execute(args=None):
 
     parser = ArgumentParser(
         "torrentfile",
+        usage="torrentfile [options] command [command options]",
         description=(
-            "Command line tools for creating, editing, checking and "
-            "interacting with Bittorrent metainfo files"
-        ),
+            "Command line tools for creating, editing, checking, building "
+            "and interacting with Bittorrent metainfo files"),
         prefix_chars="-",
         formatter_class=TorrentFileHelpFormatter,
         conflict_handler="resolve",
@@ -199,6 +230,14 @@ def execute(args=None):
         help="select program options interactively",
     )
 
+    parser.add_argument(
+        "-q",
+        "--quiet",
+        help="Turn off all text output.",
+        dest="quiet",
+        action="store_true",
+    )
+
     parser.add_argument(
         "-V",
         "--version",
@@ -218,14 +257,14 @@ def execute(args=None):
     parser.set_defaults(func=parser.print_help)
 
     subparsers = parser.add_subparsers(
-        title="Actions",
+        title="Commands",
         dest="command",
-        metavar="create, edit, magnet, recheck",
+        metavar="create, edit, info, magnet, recheck, rebuild\n",
     )
 
     create_parser = subparsers.add_parser(
         "create",
-        help="""Generate a new torrent meta file.""",
+        help="Create a new Bittorrent file.",
         prefix_chars="-",
         aliases=["c", "new"],
         formatter_class=TorrentFileHelpFormatter,
@@ -266,7 +305,6 @@ def execute(args=None):
         "--magnet",
         action="store_true",
         dest="magnet",
-        help="",
     )
 
     create_parser.add_argument(
@@ -284,7 +322,7 @@ def execute(args=None):
         action="store",
         dest="outfile",
         metavar="<path>",
-        help="Output save path for created .torrent file",
+        help="Explicitly specify the path to write the file.",
     )
 
     create_parser.add_argument(
@@ -292,14 +330,21 @@ def execute(args=None):
         "--current",
         action="store_true",
         dest="cwd",
-        help="Save output .torrent file to current directory",
+        help="*deprecated* Saving to current directory is default behaviour",
     )
 
     create_parser.add_argument(
-        "--noprogress",
-        action="store_true",
-        dest="noprogress",
-        help="Disables the progress bar during torrent creation.",
+        "--prog",
+        "--progress",
+        default="1",
+        action="store",
+        dest="progress",
+        help="""
+        Set the progress bar level.
+        Options = 0, 1
+        (0) = Do not display progress bar.
+        (1) = Display progress bar.(default)
+        """,
     )
 
     create_parser.add_argument(
@@ -311,7 +356,7 @@ def execute(args=None):
         metavar="<int>",
         help="""
         Bittorrent metafile version.
-        Options = 1, 2 or 3.
+        Options = 1, 2, 3
         (1) = Bittorrent v1 (Default)
         (2) = Bittorrent v2
         (3) = Bittorrent v1 & v2 hybrid
@@ -324,10 +369,10 @@ def execute(args=None):
         dest="piece_length",
         metavar="<int>",
         help="""
-        (Default: None) Number of bytes for each chunk of data. Acceptable
-        values include integers 14-26 or any perfect power of two between
-        16Kib and 64MiB.   (e.g. 14 and 16384 are equal)
-        Examples:: [--piece-length 14] [--piece-length 16777216]
+        (Default: <blank>) Number of bytes for per chunk of data transmitted
+        by Bittorrent client. Acceptable values include integers 14-26 which
+        will be interpreted as a perfect power of 2.  e.g. 14 = 16KiB pieces.
+        Examples:: [--piece-length 14] [--piece-length 20]
         """,
     )
 
@@ -362,9 +407,7 @@ def execute(args=None):
 
     edit_parser = subparsers.add_parser(
         "edit",
-        help="""
-        Edit existing torrent meta file.
-        """,
+        help="""Edit existing torrent meta file.""",
         aliases=["e"],
         prefix_chars="-",
         formatter_class=TorrentFileHelpFormatter,
@@ -432,11 +475,26 @@ def execute(args=None):
 
     edit_parser.set_defaults(func=edit)
 
+    info_parser = subparsers.add_parser(
+        "info",
+        help="Show detailed information about a torrent file.",
+        aliases=["i"],
+        prefix_chars="-",
+        formatter_class=TorrentFileHelpFormatter,
+    )
+
+    info_parser.add_argument(
+        "metafile",
+        action="store",
+        metavar="<*.torrent>",
+        help="path to pre-existing torrent file.",
+    )
+
+    info_parser.set_defaults(func=info)
+
     magnet_parser = subparsers.add_parser(
         "magnet",
-        help="""
-        Generate magnet url from an existing Bittorrent meta file.
-        """,
+        help="Generate magnet url from an existing Bittorrent meta file.",
         aliases=["m"],
         prefix_chars="-",
         formatter_class=TorrentFileHelpFormatter,
@@ -453,10 +511,8 @@ def execute(args=None):
 
     check_parser = subparsers.add_parser(
         "recheck",
-        help="""
-        Calculate amount of torrent meta file's content is found on disk.
-        """,
-        aliases=["r", "check"],
+        help="Gives a detailed look at how much of the torrent is available.",
+        aliases=["check"],
         prefix_chars="-",
         formatter_class=TorrentFileHelpFormatter,
     )
@@ -477,34 +533,52 @@ def execute(args=None):
 
     check_parser.set_defaults(func=recheck)
 
-    info_parser = subparsers.add_parser(
-        "info",
-        help="""
-        Show detailed information about a torrent file.
-        """,
-        aliases=["i"],
-        prefix_chars="-",
+    rebuild_parser = subparsers.add_parser(
+        "rebuild",
+        aliases=["build"],
+        help="""Re-assemble files obtained from a bittorrent file into the
+                appropriate file structure for re-seeding.  Read documentation
+                for more information, or use cases.""",
         formatter_class=TorrentFileHelpFormatter,
     )
 
-    info_parser.add_argument(
-        "metafile",
+    rebuild_parser.add_argument(
+        "metafiles",
         action="store",
         metavar="<*.torrent>",
-        help="path to pre-existing torrent file.",
+        help="path to .torrent file. or directory containing .torrent files",
     )
 
-    info_parser.set_defaults(func=info)
+    rebuild_parser.add_argument(
+        "contents",
+        action="store",
+        metavar="<contents>",
+        help="directory that contains some or all of the torrents contents",
+    )
 
+    rebuild_parser.add_argument(
+        "destination",
+        action="store",
+        metavar="<destination>",
+        help="path to where torrents will be re-assembled",
+    )
+
+    rebuild_parser.set_defaults(func=rebuild)
+    print(args)
     args = parser.parse_args(args)
 
-    if args.debug:
-        activate_logger()
+    if args.quiet:
+        Config.activate_quiet()
+
+    elif args.debug:
+        Config.activate_logger()
 
     if args.interactive:
         return select_action()
 
-    return args.func(args)
+    if hasattr(args, "func"):
+        return args.func(args)
+    return args  # pragma: nocover
 
 
 main_script = execute
