@@ -24,7 +24,8 @@ import logging
 import os
 from hashlib import sha1, sha256  # nosec
 
-from torrentfile.utils import humanize_bytes, next_power_2
+from torrentfile.mixins import CbMixin, ProgMixin
+from torrentfile.utils import next_power_2
 
 BLOCK_SIZE = 2**14  # 16KiB
 HASH_SIZE = 32
@@ -32,27 +33,7 @@ HASH_SIZE = 32
 logger = logging.getLogger(__name__)
 
 
-class _CbMixin:
-    """
-    Mixin class to set a callback during hashing procedure.
-    """
-
-    _cb = None
-
-    @classmethod
-    def set_callback(cls, func):
-        """
-        Assign a callback to the Hashing class.
-
-        Parameters
-        ----------
-        func : function
-            the callback function
-        """
-        cls._cb = func
-
-
-class Hasher(_CbMixin):
+class Hasher(CbMixin, ProgMixin):
     """
     Piece hasher for Bittorrent V1 files.
 
@@ -66,20 +47,22 @@ class Hasher(_CbMixin):
         List of files.
     piece_length : int
         Size of chuncks to split the data into.
+    progress : int
+        default = None
     """
 
-    def __init__(self, paths: list, piece_length: int):
+    def __init__(self, paths: list, piece_length: int, progress: bool = True):
         """Generate hashes of piece length data from filelist contents."""
         self.piece_length = piece_length
         self.paths = paths
-        self.total = sum([os.path.getsize(i) for i in self.paths])
+        self.progress = progress
+        self.total = sum(os.path.getsize(i) for i in self.paths)
         self.index = 0
         self.current = open(self.paths[0], "rb")
-        logger.debug(
-            "Hashing v1 torrent file. Size: %s Piece Length: %s",
-            humanize_bytes(self.total),
-            humanize_bytes(self.piece_length),
-        )
+        if self.progress:
+            total = os.path.getsize(self.paths[0])
+            self.prog_start(total, self.paths[0], unit="bytes")
+        logger.debug("Hashing %s", str(self.paths[0]))
 
     def __iter__(self):
         """
@@ -111,24 +94,40 @@ class Hasher(_CbMixin):
             temp = bytearray(target)
             size = self.current.readinto(temp)
             arr.extend(temp[:size])
+            self.prog_update(size)
             if size == target:
                 break
         return sha1(arr).digest()  # nosec
 
-    def next_file(self):
+    def next_file(self) -> bool:
         """
         Seemlessly transition to next file in file list.
+
+        Returns
+        -------
+        bool:
+            True if there is a next file otherwise False.
         """
         self.index += 1
+        self.prog_close()
         if self.index < len(self.paths):
+            path = self.paths[self.index]
+            logger.debug("Hashing %s", str(path))
             self.current.close()
-            self.current = open(self.paths[self.index], "rb")
+            if self.progress:
+                self.prog_start(os.path.getsize(path), path, unit="bytes")
+            self.current = open(path, "rb")
             return True
         return False
 
-    def __next__(self):
+    def __next__(self) -> bytes:
         """
         Generate piece-length pieces of data from input file list.
+
+        Returns
+        -------
+        bytes
+            SHA1 hash of the piece extracted.
         """
         while True:
             piece = bytearray(self.piece_length)
@@ -137,14 +136,26 @@ class Hasher(_CbMixin):
                 if not self.next_file():
                     raise StopIteration
             elif size < self.piece_length:
+                self.prog_update(size)
                 return self._handle_partial(piece[:size])
             else:
+                self.prog_update(size)
                 return sha1(piece).digest()  # nosec
 
 
 def merkle_root(blocks: list) -> bytes:
     """
     Calculate the merkle root for a seq of sha256 hash digests.
+
+    Parameters
+    ----------
+    blocks : list
+        a sequence of sha256 layer hashes.
+
+    Returns
+    -------
+    bytes
+        the sha256 root hash of the merkle tree.
     """
     if blocks:
         while len(blocks) > 1:
@@ -155,10 +166,12 @@ def merkle_root(blocks: list) -> bytes:
     return blocks
 
 
-class HasherV2(_CbMixin):
+class HasherV2(CbMixin, ProgMixin):
     """
     Calculate the root hash and piece layers for file contents.
 
+    **DEPRECATED**
+
     Iterates over 16KiB blocks of data from given file, hashes the data,
     then creates a hash tree from the individual block hashes until size of
     hashed data equals the piece-length.  Then continues the hash tree until
@@ -170,11 +183,15 @@ class HasherV2(_CbMixin):
         Path to file.
     piece_length : int
         Size of layer hashes pieces.
+    progress : int
+        default = None
     """
 
-    def __init__(self, path: str, piece_length: int):
+    def __init__(self, path: str, piece_length: int, progress: bool = True):
         """
         Calculate and store hash information for specific file.
+
+        **DEPRECATED**
         """
         self.path = path
         self.root = None
@@ -182,12 +199,8 @@ class HasherV2(_CbMixin):
         self.layer_hashes = []
         self.piece_length = piece_length
         self.num_blocks = piece_length // BLOCK_SIZE
-        logger.debug(
-            "Hashing partial v2 torrent file. Piece Length: %s Path: %s",
-            humanize_bytes(self.piece_length),
-            str(self.path),
-        )
-
+        if progress:
+            self.prog_start(os.path.getsize(path), path, unit="bytes")
         with open(self.path, "rb") as fd:
             self.process_file(fd)
 
@@ -195,9 +208,11 @@ class HasherV2(_CbMixin):
         """
         Calculate hashes over 16KiB chuncks of file content.
 
+        **DEPRECATED**
+
         Parameters
         ----------
-        fd : str
+        fd : TextIOWrapper
             Opened file in read mode.
         """
         while True:
@@ -207,6 +222,7 @@ class HasherV2(_CbMixin):
 
             for _ in range(self.num_blocks):
                 size = fd.readinto(leaf)
+                self.prog_update(size)
                 if not size:
                     break
                 blocks.append(sha256(leaf[:size]).digest())
@@ -225,6 +241,7 @@ class HasherV2(_CbMixin):
 
                 # pad the the rest with zeroes to fill remaining space.
                 padding = [bytes(32) for _ in range(remaining)]
+                self.prog_update(HASH_SIZE * remaining)
                 blocks.extend(padding)
             # calculate the root hash for the merkle tree up to piece-length
 
@@ -233,10 +250,13 @@ class HasherV2(_CbMixin):
                 self._cb(layer_hash)
             self.layer_hashes.append(layer_hash)
         self._calculate_root()
+        self.prog_close()
 
     def _calculate_root(self):
         """
         Calculate root hash for the target file.
+
+        **DEPRECATED**
         """
         self.piece_layer = b"".join(self.layer_hashes)
         hashes = len(self.layer_hashes)
@@ -249,10 +269,12 @@ class HasherV2(_CbMixin):
         self.root = merkle_root(self.layer_hashes)
 
 
-class HasherHybrid(_CbMixin):
+class HasherHybrid(CbMixin, ProgMixin):
     """
     Calculate root and piece hashes for creating hybrid torrent file.
 
+    **DEPRECATED**
+
     Create merkle tree layers from sha256 hashed 16KiB blocks of contents.
     With a branching factor of 2, merge layer hashes until blocks equal
     piece_length bytes for the piece layer, and then the root hash.
@@ -263,11 +285,15 @@ class HasherHybrid(_CbMixin):
         path to target file.
     piece_length : int
         piece length for data chunks.
+    progress : int
+        default = None
     """
 
-    def __init__(self, path: str, piece_length: int):
+    def __init__(self, path: str, piece_length: int, progress: bool = True):
         """
         Construct Hasher class instances for each file in torrent.
+
+        **DEPRECATED**
         """
         self.path = path
         self.piece_length = piece_length
@@ -277,12 +303,9 @@ class HasherHybrid(_CbMixin):
         self.root = None
         self.padding_piece = None
         self.padding_file = None
+        if progress:
+            self.prog_start(os.path.getsize(path), path, unit="bytes")
         self.amount = piece_length // BLOCK_SIZE
-        logger.debug(
-            "Hashing: %s, Piece Size: %s",
-            str(self.path),
-            humanize_bytes(self.piece_length),
-        )
         with open(path, "rb") as data:
             self.process_file(data)
 
@@ -290,6 +313,8 @@ class HasherHybrid(_CbMixin):
         """
         Generate Hash sized, 0 filled bytes for padding.
 
+        **DEPRECATED**
+
         Parameters
         ----------
         block_count : int
@@ -305,12 +330,15 @@ class HasherHybrid(_CbMixin):
         if not self.layer_hashes:
             power2 = next_power_2(block_count)
             remaining = power2 - block_count
+        self.prog_update(HASH_SIZE * remaining)
         return [bytes(HASH_SIZE) for _ in range(remaining)]
 
     def process_file(self, data: bytearray):
         """
         Calculate layer hashes for contents of file.
 
+        **DEPRECATED**
+
         Parameters
         ----------
         data : BytesIO
@@ -324,6 +352,7 @@ class HasherHybrid(_CbMixin):
             block = bytearray(BLOCK_SIZE)
             for _ in range(self.amount):
                 size = data.readinto(block)
+                self.prog_update(size)
                 if not size:
                     break
                 total += size
@@ -348,6 +377,145 @@ class HasherHybrid(_CbMixin):
                 piece.update(bytes(plength))
             self.pieces.append(piece.digest())  # nosec
         self._calculate_root()
+        self.prog_close()
+
+    def _calculate_root(self):
+        """
+        Calculate the root hash for opened file.
+
+        **DEPRECATED**
+        """
+        self.piece_layer = b"".join(self.layer_hashes)
+
+        if len(self.layer_hashes) > 1:
+            pad_piece = merkle_root([bytes(32) for _ in range(self.amount)])
+
+            pow2 = next_power_2(len(self.layer_hashes))
+            remainder = pow2 - len(self.layer_hashes)
+
+            self.layer_hashes += [pad_piece for _ in range(remainder)]
+        self.root = merkle_root(self.layer_hashes)
+
+
+class FileHasher(CbMixin, ProgMixin):
+    """
+    Calculate root and piece hashes for creating hybrid torrent file.
+
+    Create merkle tree layers from sha256 hashed 16KiB blocks of contents.
+    With a branching factor of 2, merge layer hashes until blocks equal
+    piece_length bytes for the piece layer, and then the root hash.
+
+    Parameters
+    ----------
+    path : str
+        path to target file.
+    piece_length : int
+        piece length for data chunks.
+    progress : int
+        default = None
+    """
+
+    def __init__(
+        self,
+        path: str,
+        piece_length: int,
+        progress: bool = True,
+        hybrid: bool = False,
+    ):
+        """
+        Construct Hasher class instances for each file in torrent.
+        """
+        self.path = path
+        self.piece_length = piece_length
+        self.pieces = []
+        self.layer_hashes = []
+        self.piece_layer = None
+        self.root = None
+        self.padding_piece = None
+        self.padding_file = None
+        self.amount = piece_length // BLOCK_SIZE
+        self.end = False
+        self.current = open(path, "rb")
+        self.hybrid = hybrid
+        if progress:
+            self.progressbar = True
+            self.prog_start(os.path.getsize(path), path, unit="bytes")
+
+    def __iter__(self):
+        """Return `self`: needed to implement iterator implementation."""
+        return self
+
+    def _pad_remaining(self, block_count: int):
+        """
+        Generate Hash sized, 0 filled bytes for padding.
+
+        Parameters
+        ----------
+        block_count : int
+            current total number of blocks collected.
+
+        Returns
+        -------
+        padding : bytes
+            Padding to fill remaining portion of tree.
+        """
+        # when the there is only one block for file
+        remaining = self.amount - block_count
+        if not self.layer_hashes:
+            power2 = next_power_2(block_count)
+            remaining = power2 - block_count
+        return [bytes(HASH_SIZE) for _ in range(remaining)]
+
+    def __next__(self) -> bytes:
+        """
+        Calculate layer hashes for contents of file.
+
+        Returns
+        -------
+        bytes
+            The layer merckle root hash.
+        """
+        if self.end:
+            self.end = False
+            raise StopIteration
+        plength = self.piece_length
+        blocks = []
+        piece = sha1()  # nosec
+        total = 0
+        block = bytearray(BLOCK_SIZE)
+        for _ in range(self.amount):
+            size = self.current.readinto(block)
+            if not size:
+                self.end = True
+                break
+            total += size
+            plength -= size
+            blocks.append(sha256(block[:size]).digest())
+            if self.hybrid:
+                piece.update(block[:size])
+        if len(blocks) != self.amount:
+            padding = self._pad_remaining(len(blocks))
+            blocks.extend(padding)
+        self.prog_update(total)
+        layer_hash = merkle_root(blocks)
+        self.layer_hashes.append(layer_hash)
+        if self._cb:
+            self._cb(layer_hash)
+        if self.end:
+            self._calculate_root()
+            self.prog_close()
+        if self.hybrid:
+            if plength > 0:
+                self.padding_file = {
+                    "attr": "p",
+                    "length": plength,
+                    "path": [".pad", str(plength)],
+                }
+                piece.update(bytes(plength))
+            piece = piece.digest()
+            self.pieces.append(piece)
+            return layer_hash, piece
+        return layer_hash
 
     def _calculate_root(self):
         """
@@ -363,3 +531,4 @@ class HasherHybrid(_CbMixin):
 
             self.layer_hashes += [pad_piece for _ in range(remainder)]
         self.root = merkle_root(self.layer_hashes)
+        self.current.close()
