@@ -32,13 +32,14 @@ from hashlib import sha1, sha256  # nosec
 from pathlib import Path
 
 import pyben
-from tqdm import tqdm
 
-from torrentfile.hasher import HasherHybrid, HasherV2
-from torrentfile.utils import MissingPathError
+from torrentfile.hasher import FileHasher
+from torrentfile.mixins import ProgMixin
+from torrentfile.utils import ArgumentError, MissingPathError
 
 SHA1 = 20
 SHA256 = 32
+BLOCK_SIZE = 2**14  # 16KiB
 
 logger = logging.getLogger(__name__)
 
@@ -78,16 +79,18 @@ class Checker:
         path : str
             path to content or contents parent directory.
         """
+        if not os.path.exists(metafile):
+            raise FileNotFoundError
+        if os.path.isdir(metafile):
+            raise ArgumentError(
+                "The <metafile> must be a .torrent file. Not a directory")
+        self.last_log = None
+        self.log_msg("Checking: %s, %s", metafile, path)
         self.metafile = metafile
-        self.meta_version = None
         self.total = 0
         self.paths = []
         self.fileinfo = {}
-        self.last_log = None
-
-        if not os.path.exists(metafile):
-            raise FileNotFoundError
-
+        print("Extracting data from torrent file...")
         self.meta = pyben.load(metafile)
         self.info = self.meta["info"]
         self.name = self.info["name"]
@@ -102,7 +105,6 @@ class Checker:
             self.meta_version = 1
 
         self.root = self.find_root(path)
-        self.log_msg("Checking: %s, %s", metafile, path)
         self.check_paths()
 
     @classmethod
@@ -117,21 +119,6 @@ class Checker:
         """
         cls._hook = hook
 
-    def hasher(self):
-        """
-        Return the hasher class related to torrents meta version.
-
-        Returns
-        -------
-        hasher.Hasher
-            the hashing implementation for specific torrent meta version.
-        """
-        if self.meta_version == 2:
-            return HasherV2
-        if self.meta_version == 3:
-            return HasherHybrid
-        return None
-
     def piece_checker(self):
         """
         Check individual pieces of the torrent.
@@ -149,33 +136,22 @@ class Checker:
         """
         Generate result percentage and store for future calls.
         """
-        if self.meta_version == 1:
-            iterations = len(self.info["pieces"]) // SHA1
-        else:
-            iterations = (self.total // self.piece_length) + 1
-
         responses = []
-        for response in tqdm(
-            iterable=self.iter_hashes(),
-            desc="Calculating",
-            leave=True,
-            total=iterations,
-            unit="pieces",
-        ):
+        for response in self.iter_hashes():
             responses.append(response)
 
-        self.log_msg(
-            "Final result for %s recheck:  %s", self.metafile, self._result
-        )
+        self.log_msg("Final result for %s recheck:  %s", self.metafile,
+                     self._result)
+
         return self._result
 
-    def log_msg(self, *args, level=logging.INFO):
+    def log_msg(self, *args, level: int = logging.INFO):
         """
         Log message `msg` to logger and send `msg` to callback hook.
 
         Parameters
         ----------
-        args : dict
+        *args : dict
             formatting args for log message
         level : int
             Log level for this message; default=`logging.INFO`
@@ -321,8 +297,7 @@ class Checker:
         """
         matched = consumed = 0
         checker = self.piece_checker()
-        hasher = self.hasher()
-        for chunk, piece, path, size in checker(self, hasher):
+        for chunk, piece, path, size in checker(self):
             consumed += size
             matching = 0
             if chunk == piece:
@@ -335,12 +310,11 @@ class Checker:
                 "Processed: %s%%, Matched: %s%%",
                 total_consumed,
                 percent_matched,
-                level=logging.DEBUG,
             )
         self._result = (matched / consumed) * 100 if consumed > 0 else 0
 
 
-class FeedChecker:
+class FeedChecker(ProgMixin):
     """
     Validates torrent content.
 
@@ -351,11 +325,9 @@ class FeedChecker:
     ----------
     checker : object
         the checker class instance.
-    hasher : Any
-        hashing class for calculating piece hashes. default=None
     """
 
-    def __init__(self, checker: Checker, hasher=None):
+    def __init__(self, checker: Checker):
         """
         Generate hashes of piece length data from filelist contents.
         """
@@ -363,7 +335,6 @@ class FeedChecker:
         self.paths = checker.paths
         self.pieces = checker.info["pieces"]
         self.fileinfo = checker.fileinfo
-        self.hasher = hasher
         self.piece_map = {}
         self.index = 0
         self.piece_count = 0
@@ -404,12 +375,13 @@ class FeedChecker:
         """
         partial = bytearray()
         for i, path in enumerate(self.paths):
+            total = self.fileinfo[i]["length"]
+            self.prog_start(total, path, unit="bytes")
             self.index = i
             if os.path.exists(path):
                 for piece in self.extract(path, partial):
-                    if (len(piece) == self.piece_length) or (
-                        i + 1 == len(self.paths)
-                    ):
+                    if (len(piece) == self.piece_length) or (i + 1 == len(
+                            self.paths)):
                         yield piece
                     else:
                         partial = piece
@@ -421,6 +393,7 @@ class FeedChecker:
                         yield pad
                     else:
                         partial = pad
+            self.prog_close()
 
     def extract(self, path: str, partial: bytearray) -> bytearray:
         """
@@ -452,8 +425,10 @@ class FeedChecker:
                 partial.extend(part[:amount])
                 if amount < bitlength:
                     if amount > 0 and read == length:
+                        self.prog_update(amount)
                         yield partial
                     break
+                self.prog_update(amount)
                 yield partial
                 partial = bytearray(0)
         if length != read:
@@ -492,96 +467,180 @@ class FeedChecker:
                 yield partial
 
 
-class HashChecker:
+class HashChecker(ProgMixin):
     """
-    Verify that root hashes of content files match the .torrent files.
+    Iterate through contents of meta data and verify with file contents.
 
     Parameters
     ----------
-    checker : Object
+    checker : Checker
         the checker instance that maintains variables.
-    hasher : Object
-        the version specific hashing class for torrent content.
     """
 
-    def __init__(self, checker: Checker, hasher=None):
+    def __init__(self, checker: Checker):
         """
         Construct a HybridChecker instance.
         """
         self.checker = checker
         self.paths = checker.paths
-        self.hasher = hasher
         self.piece_length = checker.piece_length
         self.fileinfo = checker.fileinfo
         self.piece_layers = checker.meta["piece layers"]
-        self.piece_count = 0
-        self.it = None
+        self.current = None
+        self.index = -1
 
     def __iter__(self):
         """
         Assign iterator and return self.
         """
-        self.it = self.iter_paths()
         return self
 
     def __next__(self):
         """
         Provide the result of comparison.
         """
+        if self.current is None:
+            self.next_file()
         try:
-            value = next(self.it)
-            return value
-        except StopIteration as stopiter:
-            raise StopIteration() from stopiter
+            return self.process_current()
+        except StopIteration as itererr:
+            if self.next_file():
+                return self.process_current()
+            raise StopIteration from itererr
 
-    def iter_paths(self) -> tuple:
+    class Padder:
         """
-        Iterate through and compare root file hashes to .torrent file.
+        Padding class to generate padding hashes wherever needed.
 
-        Yields
+        Parameters
+        ----------
+        length: int
+            the total size of the mock file generating padding for.
+        piece_length : int
+            the block size that each hash represents.
+        """
+
+        def __init__(self, length, piece_length):
+            """
+            Construct padding class to Mock missing or incomplete files.
+
+            Parameters
+            ----------
+            length : int
+                size of the file
+            piece_length : int
+                the piece length for each iteration.
+            """
+            self.length = length
+            self.piece_length = piece_length
+            self.pad = sha256(bytearray(piece_length)).digest()
+
+        def __iter__(self):
+            """
+            Return self to correctly implement iterator type.
+            """
+            return self  # pragma: nocover
+
+        def __next__(self) -> bytes:
+            """
+            Iterate through seemingly endless sha256 hashes of zeros.
+
+            Returns
+            -------
+            tuple :
+                returns the padding
+
+            Raises
+            ------
+            StopIteration
+            """
+            if self.length >= self.piece_length:
+                self.length -= self.piece_length
+                return self.pad
+            if self.length > 0:
+                pad = sha256(bytearray(self.length)).digest()
+                self.length -= self.length
+                return pad
+            raise StopIteration
+
+    def next_file(self) -> bool:
+        """
+        Remove all references to  processed files and prepare for the next.
+
+        Returns
+        -------
+        bool
+            if there is a next file found
+        """
+        self.index += 1
+        self.prog_close()
+        if self.current is None or self.index < len(self.paths):
+            self.current = self.paths[self.index]
+            self.length = self.fileinfo[self.index]["length"]
+            self.root_hash = self.fileinfo[self.index]["pieces root"]
+            if self.length > self.piece_length:
+                self.pieces = self.piece_layers[self.root_hash]
+            else:
+                self.pieces = self.root_hash
+            path = self.paths[self.index]
+            self.prog_start(self.length, path, unit="bytes")
+            self.count = 0
+            if os.path.exists(self.current):
+                self.hasher = FileHasher(path, self.piece_length, progress=0)
+            else:
+                self.hasher = self.Padder(self.length, self.piece_length)
+            return True
+        if self.index >= len(self.paths):
+            del self.current
+            del self.length
+            del self.root_hash
+            del self.pieces
+        return False
+
+    def process_current(self) -> tuple:
+        """
+        Gather necessary information to compare to metafile details.
+
+        Returns
+        -------
+        tuple
+            a tuple containing the layer, piece, current path and size
+
+        Raises
         ------
-        results : tuple
-            The size of the file and result of match.
+        StopIteration
         """
-        for i, path in enumerate(self.paths):
-            info = self.fileinfo[i]
-            length, plength = info["length"], self.piece_length
-            roothash = info["pieces root"]
-            if roothash in self.piece_layers:
-                pieces = self.piece_layers[roothash]
-            else:
-                pieces = roothash if roothash else bytes()
-            amount = len(pieces) // SHA256
+        try:
+            layer = next(self.hasher)
+            piece, size = self.advance()
+            self.prog_update(size)
+            return layer, piece, self.current, size
+        except StopIteration as err:
+            if self.length > 0 and self.count * SHA256 < len(self.pieces):
+                self.hasher = self.Padder(self.length, self.piece_length)
+                piece, size = self.advance()
+                layer = next(self.hasher)
+                self.prog_update(0)
+                return layer, piece, self.current, size
+            raise StopIteration from err
 
-            if not os.path.exists(path):
-                for i in range(amount):
-                    start = i * SHA256
-                    end = start + SHA256
-                    piece = pieces[start:end]
-                    if length > plength:
-                        size = plength
-                    else:
-                        size = length
-                    length -= size
-                    block = sha256(bytearray(size)).digest()
-                    yield block, piece, path, size
+    def advance(self) -> tuple:
+        """
+        Increment the number of pieces processed for the current file.
 
-            else:
-                hashed = self.hasher(path, plength)
-                if len(hashed.layer_hashes) == 1:
-                    block = hashed.root
-                    piece = roothash
-                    size = length
-                    yield block, piece, path, size
-                else:
-                    for i in range(amount):
-                        start = i * SHA256
-                        end = start + SHA256
-                        piece = pieces[start:end]
-                        try:
-                            block = hashed.piece_layer[start:end]
-                        except IndexError:  # pragma: nocover
-                            block = sha256(bytearray(size)).digest()
-                        size = plength if plength < length else length
-                        length -= size
-                        yield block, piece, path, size
+        Returns
+        -------
+        tuple
+            the piece and size
+        """
+        start = self.count * SHA256
+        end = start + SHA256
+        piece = self.pieces[start:end]
+        self.count += 1
+        if self.length >= self.piece_length:
+            self.length -= self.piece_length
+            size = self.piece_length
+        else:
+            size = self.length
+            self.length -= self.length
+        return piece, size
