@@ -192,10 +192,10 @@ from collections.abc import Sequence
 from datetime import datetime
 
 import pyben
-from tqdm import tqdm
 
 from torrentfile import utils
-from torrentfile.hasher import Hasher, HasherHybrid, HasherV2
+from torrentfile.hasher import FileHasher, Hasher, HasherHybrid, HasherV2
+from torrentfile.mixins import ProgMixin
 from torrentfile.version import __version__ as version
 
 logger = logging.getLogger(__name__)
@@ -221,8 +221,8 @@ class MetaFile:
         target path to write .torrent file. Default: None
     source : str
         Private tracker source. Default: None
-    noprogress : bool
-        If True disable showing the progress bar.
+    progress : str
+        level of progress bar displayed  Default: "1"
     cwd : bool
         If True change default save location to current directory
     httpseeds : list
@@ -231,6 +231,8 @@ class MetaFile:
         one or more web addressess where torrent content exists.
     content : str
         alias for 'path' arg.
+    meta_version : int
+        indicates which Bittorrent protocol to use for hashing content
     """
 
     hasher = None
@@ -257,11 +259,12 @@ class MetaFile:
         private=False,
         outfile=None,
         source=None,
-        noprogress=None,
+        progress=1,
         cwd=False,
         httpseeds=None,
         url_list=None,
         content=None,
+        meta_version=None,
         **_,
     ):
         """
@@ -270,9 +273,10 @@ class MetaFile:
         self.private = private
         self.cwd = cwd
         self.outfile = outfile
-        self.noprogress = noprogress
+        self.progress = int(progress)
         self.comment = comment
         self.source = source
+        self.meta_version = meta_version
 
         if content:
             path = content
@@ -292,11 +296,15 @@ class MetaFile:
         # base path to torrent content.
         self.path = path
 
+        logger.debug("path parameter found %s", path)
+
         # Format piece_length attribute.
         if piece_length:
             self.piece_length = utils.normalize_piece_length(piece_length)
+            logger.debug("piece length parameter found %s", piece_length)
         else:
             self.piece_length = utils.path_piece_length(self.path)
+            logger.debug("piece length calculated %s", self.piece_length)
 
         # Assign announce URL to empty string if none provided.
         if not announce:
@@ -312,22 +320,28 @@ class MetaFile:
         self.meta = {
             "announce": self.announce,
             "announce-list": self.announce_list,
-            "created by": f"TorrentFile:v{version}",
+            "created by": f"TorrentFile_v{version}",
             "creation date": int(datetime.timestamp(datetime.now())),
             "info": {},
         }
         if comment:
             self.meta["info"]["comment"] = comment
+            logger.debug("comment parameter found %s", comment)
         if private:
             self.meta["info"]["private"] = 1
+            logger.debug("private parameter triggered")
         if source:
             self.meta["info"]["source"] = source
+            logger.debug("source parameter found %s", source)
         if url_list:
             self.meta["url-list"] = url_list
+            logger.debug("url list parameter found %s", str(url_list))
         if httpseeds:
             self.meta["httpseeds"] = httpseeds
+            logger.debug("httpseeds parameter found %s", str(httpseeds))
         self.meta["info"]["piece length"] = self.piece_length
 
+        self.meta_version = meta_version
         parent, self.name = os.path.split(self.path)
         if not self.name:
             self.name = os.path.basename(parent)
@@ -346,6 +360,7 @@ class MetaFile:
 
     def sort_meta(self):
         """Sort the info and meta dictionaries."""
+        logger.debug("sorting dictionary keys")
         meta = self.meta
         meta["info"] = dict(sorted(list(meta["info"].items())))
         meta = dict(sorted(list(meta.items())))
@@ -355,6 +370,10 @@ class MetaFile:
         """
         Write meta information to .torrent file.
 
+        Final step in the torrent file creation process.
+        After hashing and sorting every piece of content
+        write the contents to file using the bencode encoding.
+
         Parameters
         ----------
         outfile : str
@@ -367,27 +386,25 @@ class MetaFile:
         meta : dict
             .torrent meta information.
         """
-        fallback = os.path.join(os.getcwd(), self.name) + ".torrent"
-        if not self.outfile and not outfile:
-            if self.cwd:
-                self.outfile = fallback
-            else:
-                path = str(self.path).rstrip("\\/")
-                self.outfile = path + ".torrent"
-        elif outfile:
+        if outfile:
             self.outfile = outfile
+        elif self.outfile:
+            pass
+        else:
+            self.outfile = os.path.join(os.getcwd(), self.name) + ".torrent"
         if str(self.outfile)[-1] in "\\/":
             self.outfile = self.outfile + (self.name + ".torrent")
         self.meta = self.sort_meta()
         try:
             pyben.dump(self.meta, self.outfile)
-        except PermissionError:
-            self.outfile = fallback
-            pyben.dump(self.meta, fallback)
+        except PermissionError as excp:
+            logger.error("Permission Denied: Could not write to %s",
+                         self.outfile)
+            raise PermissionError from excp
         return self.outfile, self.meta
 
 
-class TorrentFile(MetaFile):
+class TorrentFile(MetaFile, ProgMixin):
     """
     Class for creating Bittorrent meta files.
 
@@ -395,7 +412,7 @@ class TorrentFile(MetaFile):
 
     Parameters
     ----------
-    kwargs : dict
+    **kwargs : dict
         Dictionary containing torrent file options.
     """
 
@@ -407,11 +424,11 @@ class TorrentFile(MetaFile):
 
         Parameters
         ----------
-        kwargs : dict
+        **kwargs : dict
             dictionary of keyword args passed to superclass.
         """
         super().__init__(**kwargs)
-        logger.debug("Making Bittorrent V1 meta file.")
+        logger.debug("Assembling bittorrent v1 torrent file")
         self.assemble()
 
     def assemble(self):
@@ -428,37 +445,30 @@ class TorrentFile(MetaFile):
         if os.path.isfile(self.path):
             info["length"] = size
         else:
-            info["files"] = [
-                {
-                    "length": os.path.getsize(path),
-                    "path": os.path.relpath(path, self.path).split(os.sep),
-                }
-                for path in filelist
-            ]
-
+            info["files"] = [{
+                "length":
+                os.path.getsize(path),
+                "path":
+                os.path.relpath(path, self.path).split(os.sep),
+            } for path in filelist]
         pieces = bytearray()
-        feeder = Hasher(filelist, self.piece_length)
-        if self.noprogress:
-            for piece in feeder:
-                pieces.extend(piece)
-        else:
-            for piece in tqdm(
-                iterable=feeder,
-                desc="Torrent Pieces",
-                total=(size // self.piece_length) + 1,
-                unit="parts",
-            ):
-                pieces.extend(piece)
+
+        feeder = Hasher(filelist, self.piece_length, self.progress)
+        for piece in feeder:
+            pieces.extend(piece)
+
         info["pieces"] = pieces
 
 
-class TorrentFileV2(MetaFile):
+class TorrentFileV2(MetaFile, ProgMixin):
     """
     Class for creating Bittorrent meta v2 files.
 
+    **DEPRECATED**
+
     Parameters
     ----------
-    kwargs : dict
+    **kwargs : dict
         Keyword arguments for torrent file options.
     """
 
@@ -468,51 +478,36 @@ class TorrentFileV2(MetaFile):
         """
         Construct `TorrentFileV2` Class instance from given parameters.
 
+        **DEPRECATED**
+
         Parameters
         ----------
-        kwargs : dict
+        **kwargs : dict
             keywword arguments to pass to superclass.
         """
         super().__init__(**kwargs)
-        logger.debug("creating v2 metafile")
+        logger.debug("Assembling bittorrent v2 torrent file")
         self.piece_layers = {}
         self.hashes = []
-        self.pbar = None
         self.total = len(utils.get_file_list(self.path))
         self.assemble()
 
-    def update(self):
-        """
-        Update for the progress bar.
-        """
-        if self.pbar:
-            self.pbar.update(n=1)
-            self.total -= 1
-            if self.total <= 0:
-                del self.pbar
-                self.pbar = None
-
     def assemble(self):
         """
         Assemble then return the meta dictionary for encoding.
 
+        **DEPRECATED**
+
         Returns
         -------
         meta : dict
             Metainformation about the torrent.
         """
         info = self.meta["info"]
-        if not self.noprogress:
-            self.pbar = tqdm(
-                desc="Calculating... ",
-                total=self.total,
-                unit="files ",
-            )
-
         if os.path.isfile(self.path):
             info["file tree"] = {info["name"]: self._traverse(self.path)}
             info["length"] = os.path.getsize(self.path)
-            self.update()
+            self.prog_update(info["length"])
         else:
             info["file tree"] = self._traverse(self.path)
 
@@ -523,6 +518,8 @@ class TorrentFileV2(MetaFile):
         """
         Walk directory tree.
 
+        **DEPRECATED**
+
         Parameters
         ----------
         path : str
@@ -533,14 +530,13 @@ class TorrentFileV2(MetaFile):
             size = os.path.getsize(path)
 
             if size == 0:
-                self.update()
                 return {"": {"length": size}}
 
-            fhash = HasherV2(path, self.piece_length)
+            logger.debug("Hashing %s", str(path))
+            fhash = HasherV2(path, self.piece_length, self.progress)
 
             if size > self.piece_length:
                 self.piece_layers[fhash.root] = fhash.piece_layer
-            self.update()
             return {"": {"length": size, "pieces root": fhash.root}}
 
         file_tree = {}
@@ -550,13 +546,15 @@ class TorrentFileV2(MetaFile):
         return file_tree
 
 
-class TorrentFileHybrid(MetaFile):
+class TorrentFileHybrid(MetaFile, ProgMixin):
     """
     Construct the Hybrid torrent meta file with provided parameters.
 
+    **DEPRECATED**
+
     Parameters
     ----------
-    kwargs : dict
+    **kwargs : dict
         Keyword arguments for torrent options.
     """
 
@@ -567,16 +565,112 @@ class TorrentFileHybrid(MetaFile):
         Create Bittorrent v1 v2 hybrid metafiles.
         """
         super().__init__(**kwargs)
-        logger.debug("Creating Hybrid torrent file.")
+        logger.debug("Assembling bittorrent Hybrid file")
         self.name = os.path.basename(self.path)
         self.hashes = []
         self.piece_layers = {}
-        self.progbar = None
         self.pieces = []
         self.files = []
         self.total = len(utils.get_file_list(self.path))
         self.assemble()
 
+    def assemble(self):
+        """
+        Assemble the parts of the torrentfile into meta dictionary.
+
+        **DEPRECATED**
+        """
+        info = self.meta["info"]
+        info["meta version"] = 2
+
+        if os.path.isfile(self.path):
+            info["file tree"] = {self.name: self._traverse(self.path)}
+            info["length"] = os.path.getsize(self.path)
+
+        else:
+            info["file tree"] = self._traverse(self.path)
+            info["files"] = self.files
+
+        info["pieces"] = b"".join(self.pieces)
+        self.meta["piece layers"] = self.piece_layers
+        return info
+
+    def _traverse(self, path: str) -> dict:
+        """
+        Build meta dictionary while walking directory.
+
+        **DEPRECATED**
+
+        Parameters
+        ----------
+        path : str
+            Path to target file.
+        """
+        if os.path.isfile(path):
+            file_size = os.path.getsize(path)
+
+            self.files.append({
+                "length":
+                file_size,
+                "path":
+                os.path.relpath(path, self.path).split(os.sep),
+            })
+
+            if file_size == 0:
+                return {"": {"length": file_size}}
+
+            logger.debug("Hashing %s", str(path))
+            file_hash = HasherHybrid(path, self.piece_length, self.progress)
+            self.prog_update(file_size)
+
+            if file_size > self.piece_length:
+                self.piece_layers[file_hash.root] = file_hash.piece_layer
+
+            self.hashes.append(file_hash)
+            self.pieces.extend(file_hash.pieces)
+
+            if file_hash.padding_file:
+                self.files.append(file_hash.padding_file)
+
+            return {"": {"length": file_size, "pieces root": file_hash.root}}
+
+        tree = {}
+        if os.path.isdir(path):
+            for name in sorted(os.listdir(path)):
+                tree[name] = self._traverse(os.path.join(path, name))
+        return tree
+
+
+class TorrentAssembler(MetaFile):
+    """
+    Assembler class for Bittorrent version 2 and hybrid meta files.
+
+    This differs from the TorrentFileV2 and TorrentFileHybrid, because
+    it can be used as an iterator and works for both versions.
+
+    Parameters
+    ----------
+    **kwargs : dict
+        Keyword arguments for torrent options.
+    """
+
+    hasher = FileHasher
+
+    def __init__(self, **kwargs):
+        """
+        Create Bittorrent v1 v2 hybrid metafiles.
+        """
+        super().__init__(**kwargs)
+        logger.debug("Assembling bittorrent Hybrid file")
+        self.name = os.path.basename(self.path)
+        self.hashes = []
+        self.piece_layers = {}
+        self.pieces = bytearray()
+        self.files = []
+        self.hybrid = self.meta_version == "3"
+        self.total = len(utils.get_file_list(self.path))
+        self.assemble()
+
     def assemble(self):
         """
         Assemble the parts of the torrentfile into meta dictionary.
@@ -584,23 +678,17 @@ class TorrentFileHybrid(MetaFile):
         info = self.meta["info"]
         info["meta version"] = 2
 
-        if not self.noprogress:
-            self.progbar = tqdm(
-                desc="Calculating... ",
-                total=self.total,
-                unit="Files ",
-            )
-
         if os.path.isfile(self.path):
             info["file tree"] = {self.name: self._traverse(self.path)}
             info["length"] = os.path.getsize(self.path)
-            if self.progbar:
-                self.total -= 1
-                self.progbar.update(n=1)
+
         else:
             info["file tree"] = self._traverse(self.path)
-            info["files"] = self.files
-        info["pieces"] = b"".join(self.pieces)
+            if self.hybrid:
+                info["files"] = self.files
+
+        if self.hybrid:
+            info["pieces"] = self.pieces
         self.meta["piece layers"] = self.piece_layers
         return info
 
@@ -615,42 +703,39 @@ class TorrentFileHybrid(MetaFile):
         """
         if os.path.isfile(path):
             file_size = os.path.getsize(path)
-
-            self.files.append(
-                {
-                    "length": file_size,
-                    "path": os.path.relpath(path, self.path).split(os.sep),
-                }
-            )
+            if self.hybrid:
+                self.files.append({
+                    "length":
+                    file_size,
+                    "path":
+                    os.path.relpath(path, self.path).split(os.sep),
+                })
 
             if file_size == 0:
-                if self.progbar:
-                    self.total -= 1
-                    self.progbar.update(n=1)
                 return {"": {"length": file_size}}
 
-            file_hash = HasherHybrid(path, self.piece_length)
-
+            logger.debug("Hashing %s", str(path))
+            hasher = FileHasher(path,
+                                self.piece_length,
+                                progress=True,
+                                hybrid=self.hybrid)
+            layers = bytearray()
+            for result in hasher:
+                if self.hybrid:
+                    layer_hash, piece = result
+                    self.pieces.extend(piece)
+                else:
+                    layer_hash = result
+                layers.extend(layer_hash)
             if file_size > self.piece_length:
-                self.piece_layers[file_hash.root] = file_hash.piece_layer
+                self.piece_layers[hasher.root] = layers
+            if self.hybrid and hasher.padding_file:
+                self.files.append(hasher.padding_file)
 
-            self.hashes.append(file_hash)
-            self.pieces.extend(file_hash.pieces)
-
-            if file_hash.padding_file:
-                self.files.append(file_hash.padding_file)
-
-            if self.progbar:
-                self.total -= 1
-                self.progbar.update(n=1)
-
-            return {"": {"length": file_size, "pieces root": file_hash.root}}
+            return {"": {"length": file_size, "pieces root": hasher.root}}
 
         tree = {}
         if os.path.isdir(path):
             for name in sorted(os.listdir(path)):
                 tree[name] = self._traverse(os.path.join(path, name))
-                if self.total <= 0:
-                    del self.progbar
-                    self.progbar = None
         return tree
