@@ -40,12 +40,12 @@ from pathlib import Path
 
 class Memo:
     """
-    Memoice chache object.
+    Memoize cache.
 
     Parameters
     ----------
-    func : function
-        The function that is being memoized.
+    func : Callable
+        The results of this callable will be cached.
     """
 
     def __init__(self, func):
@@ -120,6 +120,12 @@ class PieceLengthValueError(Exception):
         super().__init__(message)
 
 
+class ArgumentError(Exception):
+    """
+    Exception for mismatched or mistyped CLI arguments.
+    """
+
+
 def humanize_bytes(amount: int) -> str:
     """
     Convert integer into human readable memory sized denomination.
@@ -160,7 +166,7 @@ def normalize_piece_length(piece_length: int) -> int:
     Raises
     ------
     PieceLengthValueError :
-        If piece length is improper value.
+        Piece length is improper value.
     """
     if isinstance(piece_length, str):
         if piece_length.isnumeric():
@@ -194,7 +200,7 @@ def get_piece_length(size: int) -> int:
         Ideal piece length.
     """
     exp = 14
-    while size / (2**exp) > 200 and exp < 25:
+    while size / (2**exp) > 1000 and exp < 24:
         exp += 1
     return 2**exp
 
@@ -283,7 +289,7 @@ def get_file_list(path: str) -> list:
 
     Returns
     -------
-    list
+    list :
         sorted list of file paths.
     """
     _, filelist = filelist_total(path)
